# $Id: hadrlib.pm,v 1.8 2006/01/11 18:10:39 uli Exp $ #
#ident "@(#)CSDBH:$RCSfile: hadrlib.pm,v $	$Revision: 1.8 $"
#
# $Copyright: Copyright (c) 2008 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
 
package hadrlib;
use strict;

use lib qw(.);
use hadrconfig() ;
use vars qw(%CONF);
*CONF = \%hadrconfig::CONF;
my $SU = $CONF{SU};
my $PASSWDFILE = $CONF{PASSWDFILE};

my $vcs_home = $ENV{"VCS_HOME"} || "/opt/VRTSvcs";
my $vcs_conf=$ENV{"VCS_CONF"} || "/etc/VRTSvcs";

my $LockFilePath = "$vcs_home" . "/bin/Db2HADR/.";
my $LockFileExt = "_LCK.dont_remove";

################# FUNCTION DEFN START ##########################

# Function : exec_db2
# Desc     : Execute a single DB2 command and analyze output
# Input    : $DB2Cmd - the db2 command including su amd env
#          : $DB2Stmt - the db2 specific function
# Output   : $DB2Out - output from the command execution
# Returns  : 0 - Output did not start with a SQL or DB2 error code
#          : 1 - output started with SQL or DB2 error code 
#
sub exec_db2 {
	my ($DB2Cmd, $DB2Stmt, $DB2Out) = @_;
	my $FirstLine = 1;

	$DB2Cmd =~ s/<DB2StmtToken>/$DB2Stmt/;
	$_[2] = `$DB2Cmd`;
	return 1 if $_[2] =~ /^SQL/;
	return 0;
}

# Function : get_hadr_config
# Desc     : Retrieve HADR state information from database configuration
# Input    : $InstOwner - owner of executables
#          : $DBName - name of the replicated database
#          : $DB2Cmd - client utility specific connectstring
# Output   : $HADRRole
#          : $RollForPend
# Returns  : $DB2Out - complete block of HADR snapshot contents
#
sub get_hadr_config {
	my ($InstOwner, $DBName, $DB2Cmd, $HADRConRole, $RollForPend) = @_;
	my ($DB2Out, $Dummy, $ret, $RetStr, @lines);
	my $DB2Stmt = "GET DATABASE CONFIGURATION FOR $DBName";

	$ret = hadrlib::exec_db2($DB2Cmd, $DB2Stmt, $DB2Out);
	return $DB2Out if $ret;

	@lines = split(/\n+/, $DB2Out);
	for (@lines) {
		if (/Rollforward pending/) {
			($Dummy, $RollForPend) = split /\s+=\s+/ if /Rollforward pending/;
			$RetStr .= "$_\n";
		}
		next unless /^\s+HADR/;
		$RetStr .= "$_\n";
		($Dummy, $HADRConRole) = split /\s+=\s+/ if /HADR database role/;
		last if /\(HADR_SYNCMODE\)/;
	}
	$_[3] = $HADRConRole;
	$_[4] = $RollForPend;
	return $RetStr;
}

# Function : get_hadr_state
# Desc     : Retrieve HADR state information from database snapshot
# Input    : $InstOwner - owner of executables
#          : $DBName - name of the replicated database
#          : $DB2Cmd - client utility specific connectstring
# Output   : $HADRRole
#          : $HADRStatus
#          : $HADRConnState
# Returns  : $DB2Out - complete block of HADR snapshot contents
#
sub get_hadr_state {
	my ($InstOwner, $DBName, $DB2Cmd, $HADRRole, $HADRStatus, $HADRConnState) = @_;
	my ($DB2Out, $Dummy, $ret, $RetStr, @lines);
	my $DB2Stmt = "GET SNAPSHOT FOR DATABASE ON $DBName";
	my $Found = 0;

	$ret = hadrlib::exec_db2($DB2Cmd, $DB2Stmt, $DB2Out);
	return $DB2Out if $ret;

	@lines = split(/\n+/, $DB2Out);
	for (@lines) {
		next unless /^HADR Status/ || $Found;
		$Found = 1;
		$RetStr .= "$_\n";
		($Dummy, $HADRRole) = split /\s+=\s+/ if /^\s+Role/;
		($Dummy, $HADRStatus) = split /\s+=\s+/ if /^\s+State/;
		($Dummy, $HADRConnState) = split /\s+=\s+/ if /^\s+Connection status/;
		last if /Log gap/;
	}
	$_[3] = $HADRRole;
	$_[4] = $HADRStatus;
	$_[5] = $HADRConnState;
	return $RetStr;
}

# Function : get_link_res_attr
# Desc     : Retrieve the instance attributes from DB2 database resource
# Input    : $LinkRes - the name of the local Db2udb resource
#          : $NodeName - the name of the local node
# Output   : $InstOwner - the User Name of the instance owner
#          : $InstHome - the home directory of the instance owner
#          : $DB2Cmd - the compound command to execute DB2 statements
# Returns  : 1 - the associated Db2udb resource or InstOwner / Insthome
#                does not exist
#
sub get_link_res_attr {
	my ($LinkRes, $NodeName, $InstOwner, $InstHome, $DB2Cmd) = @_;

	chomp($InstOwner = `$vcs_home/bin/hares -value $LinkRes DB2InstOwner $NodeName 2> /dev/null`);
	chomp($InstHome = `$vcs_home/bin/hares -value $LinkRes DB2InstHome $NodeName 2> /dev/null`);

	return 1 if $InstOwner eq "" || $InstHome eq "";

	#
	# Create db2 CLI sequence for this user
	my $UseCSH = 0;
	open (PASSWD, $PASSWDFILE);
	while (<PASSWD>) {
		next unless /^$InstOwner:/;
		$UseCSH = 1 if /csh$/;
		last;
	}
	close PASSWD;

	if ($UseCSH) {
		$DB2Cmd = "$SU $InstOwner -c \'source $InstHome/sqllib/db2cshrc; $InstHome/sqllib/bin/db2 <DB2StmtToken>\'";
	} else {
		$DB2Cmd = "$SU $InstOwner -c \'. $InstHome/sqllib/db2profile; $InstHome/sqllib/bin/db2 <DB2StmtToken>\'";
	}
	$_[2] = $InstOwner;
	$_[3] = $InstHome;
	$_[4] = $DB2Cmd;
	return 0;
}

# Function : get_local_nodename
# Desc     : Retrieve the name of the local node
# Input    : None
# Output   : None
# Returns  : Name of the local node

sub get_local_nodename {
	my $SysName = "${vcs_conf}/conf/sysname";
	my $LocSys;

	if ( -r "$SysName" ) {
		open(SNAME, "<$SysName");
	} else {
		open(SNAME, "uname -n |");
	}
	chomp($LocSys = <SNAME>);
	close SNAME;
	return $LocSys;
}

# Function : get_remote_info
# Desc     : Retrieve the name of the remote node and cluster
# Input    : $ResID - the name of the (global) resource
#          : $LocSys - the name of the local node
# Output   : $RemClust - the name of the cluster at the remote node
#          : $RemNode - the name of the remote node
# Returns  : 0 - success 
#          : 1 - one of the fields is empty

sub get_remote_info {
	my ($ResID, $LocSys, $RemClus, $RemNode) = @_;
	open(RDSP, "$vcs_home/bin/hares -display $ResID |");
	while (<RDSP>) {
		next unless /^$ResID\s+State\s+/;
		next if /$LocSys\s/;
		my @line = split();
		($RemClus, $RemNode) = split(':', $line[2]);
		if ($RemNode eq "" ) {
			#
			# This is a single cluster HADR implementation
			#
			$RemNode = $RemClus;
			$RemClus = "localclus";
		}
		last;
	}
	close RDSP;
	return 1 if ( $RemClus eq "" || $RemNode eq "" );
	$_[2] = $RemClus;
	$_[3] = $RemNode;
	return 0;
}

# Function : get_peer_res_name
# Desc     : Retrieve the name of the resource with opposite HADROper attr
# Input    : $Db2dbRes - the name of instance [resource]
#          : $DBName - the name of the database
#          : $HADROper - the operational mode of the requesting resource

# Returns  : Name of the peer resource

sub get_peer_res_name {
	my ($Db2dbRes, $DBName, $HADROper) = @_;
	my ($PeerRes, $Temp);
	if ($HADROper eq "LINK") {
		$Temp = "SWITCH";
	} else {
		$Temp = "LINK";
	}
	open(PRES, "$vcs_home/bin/hares -display -type Db2HADR -attribute ArgListValues |");
	while(<PRES>) {
		next unless /\s+$Db2dbRes\s+$DBName\s+$Temp\s+/i;
		chomp;
		($PeerRes, $Temp) = split();
		last;
	}
	close PRES;
	return $PeerRes;
}

# Function : lock_create
# Desc     : Creates a lock file to mark the resource as online
# Input    : $ResID
# Returns  : 0 - success
#
sub lock_create {
my ($ResID)=@_;

        my $LockFile = "$LockFilePath" . "$ResID" . "$LockFileExt";
        open(LCK, ">$LockFile") or return 1;
        close LCK;
        return 0;
}

# Function : lock_exists
# Desc     : Checks if the lock file for a resource exists
# Input    : $ResID
# Returns  : 0 - file does not exist
#          : 1 - file exists
#
sub lock_exists {
my ($ResID)=@_;

        my $LockFile = "$LockFilePath" . "$ResID" . "$LockFileExt";
        return 1 if -f $LockFile;
        return 0;
}

# Function : lock_remove
# Desc     : Removes the lock file for a resource to mark it offline
# Input    : $ResID
# Returns  : 0 - success
#
sub lock_remove {
my ($ResID)=@_;

        my $LockFile = "$LockFilePath" . "$ResID" . "$LockFileExt";
        unlink("$LockFile");
        return 0;
}
